import * as logger from './logger.js';
import { toString32, toHex } from './format.js';

function byteswap(a) {
  return ((a >> 24) & 0x000000ff) |
    ((a >> 8) & 0x0000ff00) |
    ((a << 8) & 0x00ff0000) |
    ((a << 24) & 0xff000000);
}

export function generateRomId(crclo, crchi) {
  return toHex(byteswap(crclo), 32) + toHex(byteswap(crchi), 32);
}

export function generateCICType(u8array) {
  let cic = 0;
  for (let i = 0; i < 0xFC0; i++) {
    cic = cic + u8array[0x40 + i];
  }

  switch (cic) {
    case 0x33a27: return '6101';
    case 0x3421e: return '6101';
    case 0x34044: return '6102';
    case 0x357d0: return '6103';
    case 0x47a81: return '6105';
    case 0x371cc: return '6106';
    case 0x343c9: return '6106';
    default:
      logger.log('Unknown CIC Code ' + toString32(cic));
      return '6102';
  }
}

export function uint8ArrayReadString(u8array, offset, maxLen) {
  let s = '';
  for (let i = 0; i < maxLen; ++i) {
    const c = u8array[offset + i];
    if (c === 0) {
      break;
    }
    s += String.fromCharCode(c);
  }
  return s;
}

export const romdb = {
  '134c3f03a7e79e31': {name:"007 - The World is Not Enough",                              save:'Eeprom4k'   },
  '9516943beb5e0af9': {name:"007 - The World is Not Enough",                              save:'Eeprom4k'   },
  '61f1ba1ff1541c2c': {name:"1080 Snowboarding",                                          save:'SRAM'       },
  '253ffd588daa2ed9': {name:"1080 Snowboarding",                                          save:'SRAM'       },
  '091da5abd9ba68c6': {name:"40 Winks",                                                   save:'Eeprom4k'   },
  'd927f2df69814d0d': {name:"A Bug's Life"                                                                  },
  'fd04dc82f4822dcf': {name:"A Bug's Life"                                                                  },
  'c0ae382b10b85063': {name:"A Bug's Life"                                                                  },
  '96c0128fe117dc45': {name:"A Bug's Life"                                                                  },
  'f18b591b459ba2ec': {name:"Aero Fighters Assault",                                      save:'Eeprom4k'   },
  '95bef662d6d602f1': {name:"Aero Fighters Assault",                                      save:'Eeprom4k'   },
  '3e46beae4b4671cc': {name:"AeroGauge",                                                  save:'Eeprom4k'   },
  '3111f480f6454638': {name:"AeroGauge",                                                  save:'Eeprom4k'   },
  'c84530d8363a9df2': {name:"AeroGauge",                                                  save:'Eeprom4k'   },
  'a682c18cb0cad0c2': {name:"AI Shogi 3"                                                                    },
  'ccffc42d215affc8': {name:"Aidyn Chronicles - The First Mage",                          save:'SRAM'       },
  '4f5aa9e623ead2ba': {name:"Aidyn Chronicles - The First Mage",                          save:'SRAM'       },
  '0cb6456c300ee5dc': {name:"Airboarder 64",                                              save:'Eeprom4k'   },
  'd025c427c1992d8c': {name:"Airboarder 64",                                              save:'Eeprom4k'   },
  '941a95b6af49c863': {name:"Akumajou Dracula Mokushiroku"                                                  },
  '063153a55b5ef2b9': {name:"Akumajou Dracula Mokushiroku Gaiden"                                           },
  'fc81607a787a8fff': {name:"Alice no Wakuwaku Trump World",                              save:'Eeprom4k'   },
  'ad84d7df036642ae': {name:"All Star Tennis '99",                                        save:'Eeprom4k'   },
  '91e285e16d76504e': {name:"All Star Tennis '99",                                        save:'Eeprom4k'   },
  '2907d2674c7796f6': {name:"All Star! Dairantou Smash Brothers"                                            },
  '4d7a545e9507e690': {name:"All-Star Baseball 2000"                                                        },
  'efc64654bb478ee1': {name:"All-Star Baseball 2001"                                                        },
  'a7233ec41a68b140': {name:"All-Star Baseball 99"                                                          },
  '4dd5edd974e2b86b': {name:"All-Star Baseball 99"                                                          },
  '0290aeb967a3b6c1': {name:"Animal Forest",                                              save:'Eeprom4k'   },
  '17dbf6c032d5e080': {name:"Armorines - Project S.W.A.R.M."                                                },
  '32d9b51f1b48a93b': {name:"Armorines - Project S.W.A.R.M."                                                },
  '5071c73c87b9cd21': {name:"Armorines - Project S.W.A.R.M."                                                },
  'b2bb524c6b0fabce': {name:"Army Men - Air Combat"                                                         },
  '57062c866d89fd8d': {name:"Army Men - Sarge's Heroes"                                                     },
  '19df10b21a8db598': {name:"Army Men - Sarge's Heroes"                                                     },
  'b6730fb234fc7529': {name:"Army Men - Sarge's Heroes 2"                                                   },
  'abd8f7d146043b29': {name:"Asteroids Hyper 64"                                                            },
  'c45db2418667721b': {name:"Automobili Lamborghini",                                     save:'Eeprom4k'   },
  'bf9777fc3ce8954a': {name:"Automobili Lamborghini",                                     save:'Eeprom4k'   },
  '9ca440e3418d3174': {name:"Baku Bomberman",                                             save:'Eeprom4k'   },
  '4f7c3ce738b893af': {name:"Baku Bomberman 2",                                           save:'Eeprom4k'   },
  '5db998df78098458': {name:"Bakuretsu Muteki Bangaioh",                                  save:'Eeprom4k'   },
  '0a98cf88b52ed58e': {name:"Bakushou Jinsei 64 - Mezase! Resort Ou"                                        },
  '20d568510dcd5fca': {name:"Banjo to Kazooie no Dai Bouken",                             save:'Eeprom4k'   },
  '00694b518198b1b4': {name:"Banjo to Kazooie no Dai Bouken 2",                           save:'Eeprom16k'  },
  'ac5975cdaef56cb2': {name:"Banjo-Kazooie",                                              save:'Eeprom4k'   },
  '0693bfa4d1df0cbf': {name:"Banjo-Kazooie",                                              save:'Eeprom4k'   },
  'b1cc3f73f9924844': {name:"Banjo-Kazooie",                                              save:'Eeprom4k'   },
  '9aaae9c2aa705d47': {name:"Banjo-Tooie",                                                save:'Eeprom16k'  },
  '396d17c9d17947ea': {name:"Banjo-Tooie",                                                save:'Eeprom16k'  },
  'df7c5b152573daf0': {name:"Banjo-Tooie",                                                save:'Eeprom16k'  },
  'b4fb88b01d4b1e44': {name:"Bass Hunter 64",                                             save:'Eeprom4k'   },
  'caa237d15350b662': {name:"Bass Tsuri No. 1 (Shigesato Itoi's Bass Fishing)"                              },
  'aaccfabcefd814b8': {name:"Bassmasters 2000"                                                              },
  'c18944202bcf8612': {name:"Batman Beyond - Return of the Joker"                                           },
  '847f9f2526ed9e7c': {name:"Batman Beyond - Return of the Joker"                                           },
  'e7dda46ae7f4e2e3': {name:"BattleTanx"                                                                    },
  '47e2a4753d960860': {name:"BattleTanx - Global Assault"                                                   },
  'e617ad0c97b7a571': {name:"BattleTanx - Global Assault"                                                   },
  'cec4d4558ac75377': {name:"Battlezone - Rise of the Black Dogs"                                           },
  '3d1c69914d5bacf4': {name:"Beast Wars Metals"                                                             },
  '16d3794d331b50e8': {name:"Beast Wars Transmetal"                                                         },
  'a819f4edcc0419bf': {name:"Beetle Adventure Racing!"                                                      },
  'd218739cc10dae24': {name:"Beetle Adventure Racing!"                                                      },
  '614ab6a10b9414d0': {name:"Beetle Adventure Racing!"                                                      },
  'b7a4ff08b653f401': {name:"Big Mountain 2000",                                          save:'Eeprom4k'   },
  '95351208def11005': {name:"Bio F.R.E.A.K.S."                                                              },
  '1d107cabb0c858ec': {name:"Bio F.R.E.A.K.S."                                                              },
  '8824ae7e5aa3409d': {name:"Bio Hazard 2",                                               save:'SRAM'       },
  'dbe6647cdb24b955': {name:"Blast Corps",                                                save:'Eeprom4k'   },
  '257c647ce601d9d9': {name:"Blast Corps (v1.0)",                                         save:'Eeprom4k'   },
  '657e647c05d34819': {name:"Blast Corps (v1.1)",                                         save:'Eeprom4k'   },
  '514423656f34d3eb': {name:"Blast Dozer",                                                save:'Eeprom16k'  },
  '128bd07c89ff5311': {name:"Blues Brothers 2000",                                        save:'SRAM'       },
  '83c871d5cf3f2d82': {name:"Blues Brothers 2000",                                        save:'SRAM'       },
  '6f692653c3999afe': {name:"Body Harvest",                                               save:'Eeprom4k'   },
  'cdb8580bd291b2b7': {name:"Body Harvest",                                               save:'Eeprom4k'   },
  'c651d4b3e258cbe1': {name:"Bokujo Monogatari 2"                                                           },
  'f4f06fdf3842d129': {name:"Bomberman 64",                                               save:'Eeprom4k'   },
  '1ed568f51eba497e': {name:"Bomberman 64",                                               save:'Eeprom4k'   },
  '3603165ab0377bbc': {name:"Bomberman 64",                                               save:'Eeprom4k'   },
  'b4737e23376b3bd6': {name:"Bomberman 64 - The Second Attack!",                          save:'Eeprom4k'   },
  'd6fd4644088278e3': {name:"Bomberman Hero",                                             save:'Eeprom4k'   },
  'cc12ff671202bf76': {name:"Bomberman Hero",                                             save:'Eeprom4k'   },
  '294e5cd8af76e288': {name:"Bomberman Hero",                                             save:'Eeprom4k'   },
  '4dd12fd7c432ed1f': {name:"Bottom of the 9th"                                                             },
  '2ecf221e13c8aa42': {name:"Brunswick Circuit Pro Bowling"                                                 },
  '1a78ae855df056c7': {name:"Buck Bumble"                                                                   },
  'b662c7d742963df8': {name:"Buck Bumble"                                                                   },
  '9c33b2d5edcabcca': {name:"Buck Bumble"                                                                   },
  '73f0868a4be545cd': {name:"Bust-A-Move 2 - Arcade Edition"                                                },
  '1ededcce02053a51': {name:"Bust-A-Move 2 - Arcade Edition"                                                },
  'fab428e3e1284a00': {name:"Bust-A-Move 3 DX"                                                              },
  '9fd8224237b6e0af': {name:"Bust-A-Move '99"                                                               },
  '0e4016ac1a075dcf': {name:"California Speed"                                                              },
  '4e2d0ff0f4aa0f34': {name:"Carmageddon 64"                                                                },
  'ec620158f18b10e3': {name:"Carmageddon 64 (censored)"                                                     },
  'f5018ee49b1fe5e6': {name:"Carmageddon 64 (uncensored)"                                                   },
  '47ffcd4b8fafa3aa': {name:"Castlevania v1.2"                                                              },
  '955f5df3693dfe8a': {name:"Castlevania"                                                                   },
  'cab7f1645537a271': {name:"Castlevania"                                                                   },
  '3863c01c26893887': {name:"Castlevania - Legacy of Darkness"                                              },
  'e74bc5a2b2cb1967': {name:"Castlevania - Legacy of Darkness"                                              },
  '3421cfdc7835d69d': {name:"Centre Court Tennis"                                                           },
  '5a53206462800250': {name:"Chameleon Twist",                                            save:'Eeprom4k'   },
  '21f5f2a48e16ebf0': {name:"Chameleon Twist",                                            save:'Eeprom4k'   },
  'c68cafb99ff1c9de': {name:"Chameleon Twist",                                            save:'Eeprom4k'   },
  'e48c53cdf9fc8a61': {name:"Chameleon Twist 2"                                                             },
  '5a76490542d0b993': {name:"Chameleon Twist 2"                                                             },
  '019da607a1417d9a': {name:"Chameleon Twist 2"                                                             },
  'e8960e1e6b82284e': {name:"Charlie Blast's Territory"                                                     },
  'd0483cfb9ff6288d': {name:"Charlie Blast's Territory"                                                     },
  '94ad4c21243b1abe': {name:"Chopper Attack",                                             save:'Eeprom4k'   },
  '3993352ea6eda53f': {name:"Chopper Attack",                                             save:'Eeprom4k'   },
  'c4f9cc2b6f9f3d40': {name:"Choro Q 64",                                                 save:'Eeprom4k'   },
  '540fcd26e0efeb53': {name:"Choro Q 64 II",                                              save:'Eeprom4k'   },
  '281594a79d19f161': {name:"Chou Snobow Kids"                                                              },
  'f40f183af78a8e5c': {name:"Chou Kuukan Night Pro Yakyuu King 2",                        save:'Eeprom4k'   },
  'b09d00f82318296b': {name:"City-Tour GP - All Japan Grand Touring Car Champion"                           },
  'ff3d5afab9cdc9b4': {name:"Clay Fighter - Sculptor's Cut"                                                 },
  'ca243cf0cc7b23c5': {name:"Clay Fighter 63 1/3"                                                           },
  'b392968e2abb6442': {name:"Clay Fighter 63 1/3"                                                           },
  'c0a76f2b2512a709': {name:"Clay Fighter 63 1/3 (beta)"                                                    },
  'ad5b02b5fd7526d3': {name:"Command & Conquer",                                          save:'FlashRam'   },
  'b46e28958fd56ab7': {name:"Command & Conquer",                                          save:'FlashRam'   },
  '65945bae76654dc5': {name:"Command & Conquer",                                          save:'FlashRam'   },
  '50acc7302d070477': {name:"Conker's Bad Fur Day",                                       save:'Eeprom16k'  },
  '89583f370aa86c9a': {name:"Conker's Bad Fur Day",                                       save:'Eeprom16k'  },
  'aff7a346d091750f': {name:"Cruis'n Exotica",                                            save:'Eeprom4k'   },
  '542540b304c04073': {name:"Cruis'n USA",                                                save:'Eeprom4k'   },
  '60a73e50960e30e1': {name:"Cruis'n USA",                                                save:'Eeprom4k'   },
  'b42f2fff9a1461d1': {name:"Cruis'n USA (v1.0)",                                         save:'Eeprom4k'   },
  '45cf06535092c4cb': {name:"Cruis'n USA (v1.1)",                                         save:'Eeprom4k'   },
  '5311e6dfe61861d7': {name:"Cruis'n World",                                              save:'Eeprom16k'  },
  '1e93f3833d2272cb': {name:"Cruis'n World",                                              save:'Eeprom16k'  },
  '870bcb835754327e': {name:"Custom Robo",                                                save:'Eeprom4k'   },
  'b9019507ab3202ab': {name:"Custom Robo V2",                                             save:'Eeprom16k'  },
  'a18efce89183739f': {name:"CyberTiger"                                                                    },
  '078aa7d13edda352': {name:"CyberTiger"                                                                    },
  '45f48871680a4184': {name:"Dance Dance Revolution - Disney Dancing Museum"                                },
  '582ba5a441987523': {name:"Dark Rift"                                                                     },
  'd47cd67e6d5e41b4': {name:"Dark Rift"                                                                     },
  '493336f51bd2f9db': {name:"Deadly Arts"                                                                   },
  'd9a9663f005bcb9b': {name:"Defi au Tetris Magique"                                                        },
  'e167f6a51fbd1fda': {name:"Derby Stallion 64",                                          save:'FlashRam'   },
  '614ac517e7f2834a': {name:"Densha de GO! 64",                                           save:'SRAM'       },
  'a284e5de8711160f': {name:"Destruction Derby 64"                                                          },
  '7da30a63dbd76b89': {name:"Destruction Derby 64"                                                          },
  '9c167989a0f689f1': {name:"Dezaemon 3D",                                                save:'SRAM'       },
  '725629d51570b59c': {name:"Diddy Kong CRACKED",                                         save:'Eeprom4k'   },
  '5aa389f362557817': {name:"Diddy Kong Racing",                                          save:'Eeprom4k'   },
  'e740d45311b01975': {name:"Diddy Kong Racing (v1.0)",                                   save:'Eeprom4k'   },
  '75f773fd5a752497': {name:"Diddy Kong Racing (v1.0)",                                   save:'Eeprom4k'   },
  '0d4302e49dfcfcd2': {name:"Diddy Kong Racing (v1.1)",                                   save:'Eeprom4k'   },
  '5b146e599f87d9f7': {name:"Diddy Kong Racing (v1.1)",                                   save:'Eeprom4k'   },
  '1b426cc1f78015a2': {name:"Disney's Donald Duck - Goin' Quackers",                      save:'Eeprom4k'   },
  '2313264c1a5e294f': {name:"Disney's Tarzan"                                                               },
  'c769fecb2aabc0f2': {name:"Disney's Tarzan",                                            save:'Eeprom4k'   },
  'd03b1a001adeb3af': {name:"Disney's Tarzan"                                                               },
  'bfe514d6c1bc6da7': {name:"Disney's Tarzan",                                            save:'Eeprom4k'   },
  '6d208ebd1c5ec398': {name:"Doubutsu No Mori",                                           save:'Eeprom4k'   },
  '8074f13d5aed3d19': {name:"Donald Duck - Quack Attack"                                                    },
  'bfea58ec69717cad': {name:"Donkey Kong 64",                                             save:'Eeprom16k'  },
  'a7893c05024306a5': {name:"Donkey Kong 64",                                             save:'Eeprom16k'  },
  '8c6d9311434b2c6f': {name:"Donkey Kong 64",                                             save:'Eeprom16k'  },
  'ababd40d1ea9a2b5': {name:"Donkey Kong 64 - Kiosk",                                     save:'Eeprom16k'  },
  '1a103ea89db637e9': {name:"Doom 64"                                                                       },
  'f4963e425bf088ce': {name:"Doom 64"                                                                       },
  '365ba67aade5cefd': {name:"Doom 64"                                                                       },
  'ac9e732c2677f79e': {name:"Doom 64"                                                                       },
  'c2b1f7bf8e14bfae': {name:"Doraemon - Mittsu no Seireiseki",                            save:'Eeprom4k'   },
  '996e30b6b2d23eb6': {name:"Doraemon 2 - Hikari no Shinden",                             save:'Eeprom16k'  },
  '405127a8e856b0b9': {name:"Doraemon 3 - Nobi Dai No Machi SOS!",                        save:'Eeprom16k'  },
  '134d9d76fe3f23da': {name:"Dr. Mario 64",                                               save:'Eeprom4k'   },
  'a0ecde3fb29b9f39': {name:"Dragon Sword 64"                                                               },
  'c33022a6884483f0': {name:"Dual Heroes"                                                                   },
  '63ab6e05d5fc15c2': {name:"Dual Heroes"                                                                   },
  '614452b6b1046ded': {name:"Dual Heroes"                                                                   },
  'faf1b9fb8986f86b': {name:"Duck Dodgers Starring Daffy Duck",                           save:'Eeprom4k'   },
  '7ff0da0488e6180d': {name:"Duke Nukem - ZER0 H0UR"                                                        },
  '4b97ca32509cc2b2': {name:"Duke Nukem - ZER0 H0UR"                                                        },
  '6a6236dccb70373f': {name:"Duke Nukem - ZER0 H0UR"                                                        },
  '56ab73a29adb33da': {name:"Duke Nukem 64"                                                                 },
  '3d88121e1827b9d3': {name:"Duke Nukem 64"                                                                 },
  '4df7bf57437774de': {name:"Duke Nukem 64"                                                                 },
  'cfb6db349d7806ea': {name:"Earthworm Jim 3D Ru",                                        save:'Eeprom4k'   },
  '914157df3d12b59e': {name:"Earthworm Jim 3D",                                           save:'Eeprom4k'   },
  'e89d2b491cc8ccc6': {name:"Earthworm Jim 3D",                                           save:'Eeprom4k'   },
  '6d76f9bd708d06bd': {name:"ECW Hardcore Revolution"                                                       },
  'dbe5388cd7277cb3': {name:"ECW Hardcore Revolution"                                                       },
  'cb53a6f23b3b6360': {name:"Elmo's Letter Adventure"                                                       },
  '8f53b102d0884bc9': {name:"Elmo's Number Journey"                                                         },
  'dce23ae1e85cb64f': {name:"Eltale Monsters"                                                               },
  '421886079fbc2ea1': {name:"Excitebike 64",                                              save:'Eeprom16k'  },
  '19351c86e51c09f6': {name:"Excitebike 64",                                              save:'Eeprom16k'  },
  'e48e2a20898bf883': {name:"Excitebike 64",                                              save:'Eeprom16k'  },
  '7b4f75af8173d11d': {name:"Excitebike 64 - Kiosk",                                      save:'Eeprom16k'  },
  'd245a2fd473d4aa7': {name:"Extreme-G"                                                                     },
  'c42d80ee7dd50b69': {name:"Extreme-G"                                                                     },
  '4e839d8ea9298b1e': {name:"Extreme-G"                                                                     },
  '0b15d45cf1c20c47': {name:"Extreme-G XG2"                                                                 },
  '819b9b3911ad33d5': {name:"Extreme-G XG2"                                                                 },
  '85ec851131775a4b': {name:"Extreme-G XG2"                                                                 },
  '7a6882ae8d383f9a': {name:"F-1 Pole Position 64"                                                          },
  'b248d2fd0e029a56': {name:"F-1 Pole Position 64"                                                          },
  'b8bcec3c07bf2661': {name:"F1 Racing Championship"                                                        },
  '3426443860f0b366': {name:"F-1 World Grand Prix",                                       save:'Eeprom4k'   },
  'b3c83ccca405c40e': {name:"F-1 World Grand Prix",                                       save:'Eeprom4k'   },
  'e5ae0bb7a805503a': {name:"F-1 World Grand Prix",                                       save:'Eeprom4k'   },
  'c447bf64ba22bdf4': {name:"F-1 World Grand Prix",                                       save:'Eeprom4k'   },
  '6e86c107decc7557': {name:"F-1 World Grand Prix II",                                    save:'Eeprom16k'  },
  '374cff6dfd63b7b1': {name:"Famista 64"                                                                    },
  '4f47294f7a70cb30': {name:"F-Cup Maniax"                                                                  },
  'decd1acbf21d29cf': {name:"FIFA - Road to World Cup 98"                                                   },
  '673c73f53a97a317': {name:"FIFA - Road to World Cup 98"                                                   },
  'f0ed310ed54972c3': {name:"FIFA - Road to World Cup 98"                                                   },
  '30a61376f396d63e': {name:"FIFA 99"                                                                       },
  '51a69801849d21fc': {name:"FIFA 99"                                                                       },
  '5991f1c35abcd265': {name:"FIFA Soccer 64"                                                                },
  '70e594fe9c3a87e4': {name:"Fighter's Destiny",                                          save:'Eeprom4k'   },
  '0588f752b7ca8f8b': {name:"Fighter's Destiny",                                          save:'Eeprom4k'   },
  'c2f9410c0d7a7101': {name:"Fighter's Destiny",                                          save:'Eeprom4k'   },
  '4bc7f136399902f2': {name:"Fighter's Destiny",                                          save:'Eeprom4k'   },
  '452fefaef1307ef9': {name:"Fighter's Destiny 2",                                        save:'Eeprom4k'   },
  '2d6ce4490c111a7b': {name:"Fighting Cup"                                                                  },
  'cbc7ef32203feac3': {name:"Fighting Force 64"                                                             },
  'fe0fcf669c7f69ad': {name:"Fighting Force 64"                                                             },
  '2d56d52850aed5e4': {name:"Fire Electric Pen",                                          save:'Eeprom4k'   },
  'e5522da955b6261d': {name:"Flying Dragon"                                                                 },
  '3f62e922ad520eb6': {name:"Flying Dragon"                                                                 },
  '1a8fff35bee3796e': {name:"Flying Dragon Fighters (Hiro no Ken Twin)"                                     },
  '79b1e5e8e830aa44': {name:"Frogger 2 (Alpha)"                                                             },
  'ff76cdc3decb9d9b': {name:"Forsaken"                                                                      },
  '010c339eba14038c': {name:"Forsaken"                                                                      },
  'aa172a14968d0213': {name:"Forsaken"                                                                      },
  '79d4613225bc0ded': {name:"Fox Sports College Hoops '99"                                                  },
  'eeea74f73eb1d8f0': {name:"Fushigi no Dungeon - Furai no Shiren 2",                     save:'FlashRam'   },
  'a9d183398c150420': {name:"F-Zero DXP",                                                 save:'SRAM'       },
  '78d90eb3f9c90330': {name:"F-Zero X",                                                   save:'SRAM'       },
  '2e623e4d4e8b829b': {name:"F-Zero X",                                                   save:'SRAM'       },
  'f64666772bacb906': {name:"F-Zero X",                                                   save:'SRAM'       },
  'b67986afbf11105e': {name:"G.A.S.P! Fighter's NEXTream",                                save:'Eeprom16k'  },
  '26f7fc68bc8c6549': {name:"G.A.S.P! Fighter's NEXTream",                                save:'Eeprom16k'  },
  '8b162c83aecda256': {name:"Ganbare Goemon"                                                                },
  'd99c7b455253c509': {name:"Ganbare Goemon 2 - Deoru Dero Douchuu Obake Tenko"                             },
  '0afdff21981d0dfa': {name:"Ganbare Nippon Olympics 2000"                                                  },
  '325e9b7280d928b7': {name:"Gauntlet Legends"                                                              },
  '0e26b0704cd01667': {name:"Gauntlet Legends"                                                              },
  'd6bc43d556e2a52b': {name:"Gauntlet Legends"                                                              },
  'e6849c48f9496e4c': {name:"Getter Love!! Cho Renai Party Game"                                            },
  '127edc3ec91c6ce2': {name:"Gex 3 - Deep Cover Gecko"                                                      },
  '59931799c3ebe72f': {name:"Gex 3 - Deep Cover Gecko"                                                      },
  'a43347875a7423a8': {name:"Gex 3 - Deep Cover Gecko"                                                      },
  '74d7fe891be2afca': {name:"Gex 64 - Enter the Gecko"                                                      },
  '0e008ae6dd669163': {name:"Gex 64 - Enter the Gecko"                                                      },
  'ff016e8e48f9b4cc': {name:"Glover",                                                     save:'Eeprom4k'   },
  '017323f593eee399': {name:"Glover",                                                     save:'Eeprom4k'   },
  '7fd2c6b2fd8ca42d': {name:"Goemon Mononoke Sugoroku"                                                      },
  'ada552424ebf6fae': {name:"Goemon's Great Adventure"                                                      },
  '1cfb9046446dd54c': {name:"Golden Nugget 64"                                                              },
  'd150bcdca31afd09': {name:"GoldenEye 007",                                              save:'Eeprom4k'   },
  'f14c4fa2ba2723a8': {name:"GoldenEye 007",                                              save:'Eeprom4k'   },
  '61ca1404aab8572e': {name:"GoldenEye 007",                                              save:'Eeprom4k'   },
  'a2dc9ac4621b50f1': {name:"GT 64 Championship Edition",                                 save:'Eeprom4k'   },
  '330e4aeec988d58f': {name:"GT 64 Championship Edition",                                 save:'Eeprom4k'   },
  '1401a8957f2ab7e0': {name:"Hamster Monogatari 64",                                      save:'Eeprom4k'   },
  'bf2ff236f2128931': {name:"Hanafuda 64 - Tenshi no Yakusoku"                                              },
  'b157ae0937562a18': {name:"Harukanaru Augusta Masters 98",                              save:'Eeprom4k'   },
  'fc9ddf9889c10666': {name:"Harvest Moon 64",                                            save:'SRAM'       },
  '66e8703ee8ba3844': {name:"Heiwa Pachinko World",                                       save:'Eeprom4k'   },
  '77eb3c7f0a038189': {name:"Hercules - The Legendary Journeys"                                             },
  'ebdb90ae2391b879': {name:"Hercules - The Legendary Journeys"                                             },
  '0ab5b39a056166bc': {name:"Hexen"                                                                         },
  'bd5f1b5c3416967e': {name:"Hexen"                                                                         },
  'ea6aab9c001cc687': {name:"Hexen"                                                                         },
  '571a75666e9da254': {name:"Hexen"                                                                         },
  '0bb3b295c115642b': {name:"Hexen"                                                                         },
  '5d0ef1d379a52e05': {name:"Hey You, Pikachu!",                                          save:'Eeprom4k'   },
  '140efa7505d1b3c9': {name:"Holy Magic Century"                                                            },
  'b0eb5fb304b22774': {name:"Holy Magic Century"                                                            },
  '9d127b27ff7938dd': {name:"Holy Magic Century"                                                            },
  'bd02d7c14765416d': {name:"Hoshi no Kirby 64",                                          save:'Eeprom4k'   },
  '8e8fc9c7de5d1442': {name:"Hot Wheels Turbo Racing"                                                       },
  '9301d2e7938e15c1': {name:"Hot Wheels Turbo Racing"                                                       },
  '7d1d6172d2bd1999': {name:"HSV Adventure Racing"                                                          },
  '2e97355595328ebd': {name:"Human Grand Prix"                                                              },
  '6ac9dbe8eee2c9da': {name:"Hybrid Heaven"                                                                 },
  'bf882810ca884843': {name:"Hybrid Heaven"                                                                 },
  '36cee20de6291dd4': {name:"Hybrid Heaven"                                                                 },
  '7f3a1d6466048286': {name:"Hybrid Heaven"                                                                 },
  'eb65dcc804898c3d': {name:"Hydro Thunder"                                                                 },
  'e98889b5e84bfcb1': {name:"Hydro Thunder"                                                                 },
  '9e8b456936f995fc': {name:"Iggy-kun no Bura Bura Poyon"                                                   },
  'bcb516e6888b65c9': {name:"Iggy's Reckin' Balls"                                                          },
  '5ecc92d672d058ec': {name:"Iggy's Reckin' Balls"                                                          },
  '15cc9daf883d721a': {name:"Indiana Jones and the Infernal Machine",                     save:'Eeprom4k'   },
  '7a4636e49b8fde82': {name:"Indy Racing 2000",                                           save:'Eeprom4k'   },
  'e08b138c460e7095': {name:"In-Fisherman Bass Hunter 64",                                save:'Eeprom4k'   },
  '3754838eb44857cd': {name:"International Superstar Soccer 2000"                                           },
  'a0646333bfd5c806': {name:"International Superstar Soccer 2000"                                           },
  '71e8e8ba4e94ff35': {name:"International Superstar Soccer 2000"                                           },
  'c463275fe52a4162': {name:"International Superstar Soccer 64"                                             },
  'f07cd3e2ae4e7ef5': {name:"International Superstar Soccer 64"                                             },
  '43631bf4e66106c1': {name:"International Superstar Soccer 98"                                             },
  '09da0f7f0bce6160': {name:"International Superstar Soccer '98"                                            },
  'c73b072011013b5e': {name:"International Track and Field 2000"                                            },
  '79c712671d78723b': {name:"International Track and Field Summer Games"                                    },
  '476776876571c291': {name:"J. League Dynamite Soccer"                                                     },
  'aa0acec619f017d1': {name:"J. League Tactics Soccer (1.1)"                                                },
  '29a4bf4f15bb2069': {name:"J. League Eleven Beat 1997"                                                    },
  '424a5554fb5f98e4': {name:"J. League Live 64"                                                             },
  'a09dd2e8941de615': {name:"J. League Tactics Soccer"                                                      },
  '16d03ac77d53c548': {name:"Jangou Simulation Mahjong Do 64"                                               },
  '6064256986f5a3b9': {name:"Jeopardy!"                                                                     },
  'a5b130bbce12f7fc': {name:"Jeremy McGrath Supercross 2000"                                                },
  'fbabf721e8a78a6a': {name:"Jeremy McGrath Supercross 2000"                                                },
  'b609608a50e1ac94': {name:"Jet Force Gemini",                                           save:'FlashRam'   },
  'dea1d7684a837900': {name:"Jet Force Gemini",                                           save:'FlashRam'   },
  '47abd8df89ebdb3c': {name:"Jet Force Gemini - Kiosk",                                   save:'FlashRam'   },
  'f5d919afcc2302b7': {name:"Jikkyou G1 Stable"                                                             },
  '532a11638fa89fa2': {name:"Jikkyou J. League 1999 - Perfect Striker 2"                                    },
  '66436c14b3dea672': {name:"Jikkyou J. League Perfect Striker"                                             },
  '1c1f89d78837e4c3': {name:"Jikkyou Pawafuru Puroyakyu 4 (1.1)"                                            },
  'd144c20a05c60e1f': {name:"Jikkyou Pawapuro Puroyakyu 2000"                                               },
  '23df6442f7bd28be': {name:"Jikkyou Pawapuro Puroyakyu 2000"                                               },
  '2056ea3f40db5674': {name:"Jikkyou World Cup France '98"                                                  },
  '8cb354c9b3be626f': {name:"Jikkyou World Cup France '98"                                                  },
  'd6abc7e1287f704e': {name:"Jikkyou World Cup France '98"                                                  },
  '8c9fa7e0fa97cc32': {name:"Jikkyou World Soccer 3"                                                        },
  'edf6aa4aad286437': {name:"Jinsei Game 64"                                                                },
  'b01a15d04ba15cfe': {name:"John Romero's Daikatana"                                                       },
  '4b3c7e9d6c4a0fe6': {name:"John Romero's Daikatana"                                                       },
  '9531740f95dba6d8': {name:"John Romero's Daikatana"                                                       },
  '231f2836cf569700': {name:"Ken Griffey Jr.'s Slugfest",                                 save:'FlashRam'   },
  '3e269b97040047f8': {name:"Killer Instinct Gold",                                       save:'Eeprom4k'   },
  'bae28f9e7007278b': {name:"Killer Instinct Gold (v1.0)",                                save:'Eeprom4k'   },
  'facd8f9e2b65f549': {name:"Killer Instinct Gold (v1.0)",                                save:'Eeprom4k'   },
  '4cca08f927434636': {name:"Killer Instinct Gold (v1.2)",                                save:'Eeprom4k'   },
  'e1a49e51e88475eb': {name:"King Hill 64 - Extreme Snowboarding"                                           },
  'd66abc75c92b5578': {name:"Kiratto Kaiketsu! 64 Tanteidan"                                                },
  'b49f03462c823703': {name:"Kirby 64 - The Crystal Shards",                              save:'Eeprom4k'   },
  '11ba930da6683868': {name:"Kirby 64 - The Crystal Shards",                              save:'Eeprom4k'   },
  '6fb81bcac5a5cc41': {name:"Kirby 64 - The Crystal Shards (1.1)",                        save:'Eeprom4k'   },
  '7a1c580ce4206e3d': {name:"Kirby 64 - The Crystal Shards (1.2)",                        save:'Eeprom4k'   },
  '9ff8b1bca2520706': {name:"Kirby 64 - The Crystal Shards (1.3)",                        save:'Eeprom4k'   },
  '99d7e0fc546c3165': {name:"Knife Edge - Nose Gunner"                                                      },
  '3fef1a93906b19ef': {name:"Knife Edge - Nose Gunner"                                                      },
  '747c994a997f08e2': {name:"Knife Edge - Nose Gunner"                                                      },
  '9c9094082dd8d4da': {name:"Knockout Kings 2000"                                                           },
  '95a7d6e33c5d1c2a': {name:"Knockout Kings 2000"                                                           },
  '94846b611092508a': {name:"Kobe Bryant's NBA Courtside",                                save:'Eeprom16k'  },
  'baef3917683ab4d0': {name:"Kobe Bryant's NBA Courtside",                                save:'Eeprom16k'  },
  'e8603e5e95d4b54a': {name:"Kuiki Uhabi Suigo"                                                             },
  '9940307f7652cf52': {name:"Last Legion UX",                                             save:'Eeprom4k'   },
  'ea406a09100abe8a': {name:"LEGO Racers"                                                                   },
  'b3d878f46ddd1697': {name:"LEGO Racers"                                                                   },
  '2bc6673d5031d031': {name:"Let's Smash Tennis"                                                            },
  'df1850253aaed657': {name:"Lode Runner 3-D",                                            save:'Eeprom4k'   },
  '0bdd4a96db1392b2': {name:"Lode Runner 3-D",                                            save:'Eeprom4k'   },
  '80064660720e5f30': {name:"Lode Runner 3-D",                                            save:'Eeprom4k'   },
  '5b05a00a65df3776': {name:"Looney Tunes - Duck Dodgers",                                save:'Eeprom4k'   },
  '2ce9cbf412ed92b3': {name:"Lylat Wars",                                                 save:'Eeprom4k'   },
  '2bf283245e026e13': {name:"Lylat Wars",                                                 save:'Eeprom4k'   },
  '5007706bfe21d629': {name:"Mace - The Dark Age"                                                           },
  '3d444511db0e6111': {name:"Mace - The Dark Age"                                                           },
  '8616b80c815ad85f': {name:"Madden 2000"                                                                   },
  '89638313763c5b26': {name:"Madden Football 64"                                                            },
  '52cb97a10ede2075': {name:"Madden Football 64"                                                            },
  '8d4f13d7b5001ac1': {name:"Madden NFL 2002",                                            save:'Eeprom16k'  },
  '92f738eb46a20e19': {name:"Madden NFL 2001"                                                               },
  'ba8bb7de9dbdf652': {name:"Madden NFL 99"                                                                 },
  '25d625395ec7838c': {name:"Madden NFL 99"                                                                 },
  'f793efe10b8b9014': {name:"Magical Tetris Challenge"                                                      },
  '4716b67578bfda7a': {name:"Magical Tetris Challenge"                                                      },
  '4a56c880ab659c92': {name:"Magical Tetris Challenge"                                                      },
  '796690e4053f249f': {name:"Magical Tetris Challenge"                                                      },
  '41dc3ec55993e1ec': {name:"Mahjong 64"                                                                    },
  '1e82cccc838ae896': {name:"Mahjong Hourouki Classic"                                                      },
  '702cc40fcdf15487': {name:"Mahjong Master"                                                                },
  '5cc0c180f45e06ea': {name:"Major League Baseball Featuring Ken Griffey Jr.",            save:'SRAM'       },
  'be98b9cdc8a52410': {name:"Major League Baseball Featuring Ken Griffey Jr.",            save:'SRAM'       },
  'd4a34b66b7808a67': {name:"Mario Golf 64",                                              save:'SRAM'       },
  'd14489d40e3ad9b0': {name:"Mario Golf 64",                                              save:'SRAM'       },
  'd057e9625d5ac17f': {name:"Mario Golf 64",                                              save:'SRAM'       },
  'b655503e52da922e': {name:"Mario Kart 64",                                              save:'Eeprom4k'   },
  '5847ff6b5e5dffe5': {name:"Mario Kart 64 (v1.0)",                                       save:'Eeprom4k'   },
  '9ddeb6c376ded265': {name:"Mario Kart 64 (v1.0)",                                       save:'Eeprom4k'   },
  '87a9c3c94c341058': {name:"Mario Kart 64 (v1.1)",                                       save:'Eeprom4k'   },
  'd4c77725aeaa8fd1': {name:"Mario Kart 64 (v1.1)",                                       save:'Eeprom4k'   },
  'ac90989adf13c3f0': {name:"Mario No Photopie"                                                             },
  '7e652928771862a0': {name:"Mario Party",                                                save:'Eeprom4k'   },
  'be15a8ad2f622860': {name:"Mario Party",                                                save:'Eeprom4k'   },
  '6930669c804af280': {name:"Mario Party",                                                save:'Eeprom4k'   },
  '5858a99e18b672af': {name:"Mario Party 2",                                              save:'Eeprom4k'   },
  '0f7d56ed1589b038': {name:"Mario Party 2",                                              save:'Eeprom4k'   },
  '87033882d944c7df': {name:"Mario Party 2",                                              save:'Eeprom4k'   },
  'd929387cce47826e': {name:"Mario Party 3",                                              save:'Eeprom16k'  },
  'cdb40a0b3789157b': {name:"Mario Party 3",                                              save:'Eeprom16k'  },
  '604167c53c455f0f': {name:"Mario Party 3",                                              save:'Eeprom16k'  },
  'dccda73ba0524e46': {name:"Mario Story",                                                save:'FlashRam'   },
  '4fcf0150bdb30cf3': {name:"Mario Tennis",                                               save:'Eeprom16k'  },
  'b5426c3a1bdaca1a': {name:"Mario Tennis",                                               save:'Eeprom16k'  },
  'd53a9f83fa156d40': {name:"Mario Tennis",                                               save:'Eeprom16k'  },
  'f558c10e96683efb': {name:"Mega Man 64",                                                save:'FlashRam'   },
  '0cf10110c1d8513d': {name:"Mia Hamm Soccer 64"                                                            },
  'c26661e3e5a21386': {name:"Michael Owens WLS 2000"                                                        },
  '71458cfac0f9e7bb': {name:"Mickey's Speedway USA",                                      save:'Eeprom4k'   },
  '9addd0dea72582e7': {name:"Mickey's Speedway USA",                                      save:'Eeprom4k'   },
  'afe66a73c7e91741': {name:"Mickey no Racing Challenge USA",                             save:'Eeprom4k'   },
  '350c85f11279e0ac': {name:"Micro Machines 64 Turbo"                                                       },
  '8d01492a024a03d0': {name:"Micro Machines 64 Turbo"                                                       },
  '4c5eb3e4c95cc41a': {name:"Midway's Greatest Arcade Hits Volume 1"                                        },
  '163ed509b968b23a': {name:"Mike Piazza's Strike Zone"                                                     },
  'cd5e952e840800f3': {name:"Milo's Astro Lanes"                                                            },
  '9d0d499adc3a018f': {name:"Milo's Astro Lanes"                                                            },
  '1b05930bf9813d60': {name:"Mischief Makers",                                            save:'Eeprom4k'   },
  '98da8b41580f8a24': {name:"Mischief Makers",                                            save:'Eeprom4k'   },
  '7e3feb93445e6781': {name:"Mission Impossible",                                         save:'Eeprom4k'   },
  'f85c032635912b80': {name:"Mission Impossible",                                         save:'Eeprom4k'   },
  '345b0920879e3d34': {name:"Mission Impossible",                                         save:'Eeprom4k'   },
  'dc49a9ebbdecba39': {name:"Mission Impossible",                                         save:'Eeprom4k'   },
  'daec56229c1bab71': {name:"Mission Impossible",                                         save:'Eeprom4k'   },
  'e2046a5f0d07fad4': {name:"Mission Impossible",                                         save:'Eeprom4k'   },
  '6d8d76286c9779b3': {name:"Monaco Grand Prix",                                          save:'Eeprom4k'   },
  'e183c35a87e312d7': {name:"Monopoly",                                                   save:'Eeprom4k'   },
  '99d99ab11851587e': {name:"Monster Truck Madness 64"                                                      },
  'fc06d8d3a8a23ab4': {name:"Monster Truck Madness 64"                                                      },
  '70dde8e898d15d41': {name:"Morita Shogi 64",                                            save:'Eeprom4k'   },
  'f4d47d41e22f481b': {name:"Mortal Kombat 4",                                            save:'Eeprom4k'   },
  '3b6f0373e9690dce': {name:"Mortal Kombat 4",                                            save:'Eeprom4k'   },
  'ac0443c321c0792d': {name:"Mortal Kombat Mythologies - Sub-Zero"                                          },
  'c4ed44ff1392ae1a': {name:"Mortal Kombat Mythologies - Sub-Zero"                                          },
  '125cf7d9599b85a8': {name:"Mortal Kombat Trilogy"                                                         },
  '92113d8ce172f1be': {name:"Mortal Kombat Trilogy"                                                         },
  'a93af3830dd401a9': {name:"Mortal Kombat Trilogy (v1.2)"                                                  },
  '5cb6f92ad7a2e285': {name:"MRC - Multi Racing Championship",                            save:'Eeprom4k'   },
  '13b4b6a6cc13d115': {name:"MRC - Multi Racing Championship",                            save:'Eeprom4k'   },
  '03bdf0b89e187944': {name:"MRC - Multi Racing Championship",                            save:'Eeprom4k'   },
  '5c52381956966e58': {name:"Ms. Pac-Man Maze Madness"                                                      },
  '21cbbcfc6b3c9072': {name:"Mystical Ninja - Starring Goemon"                                              },
  'be0f36f51d69f12b': {name:"Mystical Ninja - Starring Goemon"                                              },
  'd345937fdeca1e84': {name:"Mystical Ninja 2 - Starring Goemon"                                            },
  '8725c27e23e31aef': {name:"Nagano Olympic Hockey '98"                                                     },
  '3730f490f570535c': {name:"Nagano Olympic Hockey '98"                                                     },
  '353a2dae1ad4f024': {name:"Nagano Olympic Hockey '98"                                                     },
  '98ae2b8dbf2537d7': {name:"Nagano Winter Olympics '98"                                                    },
  '4cc3c52f9dcc057a': {name:"Nagano Winter Olympics '98"                                                    },
  '1620456dee093c71': {name:"Nagano Winter Olympics '98"                                                    },
  'dab629518c3cef9d': {name:"Namco Museum 64"                                                               },
  '181a33df44e0d45f': {name:"NASCAR 2000"                                                                   },
  '78957423fd58dc80': {name:"NASCAR 99"                                                                     },
  '49140000c99249ae': {name:"NASCAR 99"                                                                     },
  'c99249ae53b25392': {name:"NASCAR 99"                                                                     },
  '57ebf724f4f9b374': {name:"NBA Courtside 2 - Featuring Kobe Bryant",                    save:'FlashRam'   },
  'd8526891efeadb73': {name:"NBA Courtside 2 - Featuring Kobe Bryant",                    save:'FlashRam'   },
  '87b4694e90e218fe': {name:"NBA Hangtime"                                                                  },
  'aedc88c70a0003bd': {name:"NBA Hangtime"                                                                  },
  '011fe1aa45a02526': {name:"NBA In the Zone 2000"                                                          },
  '185bf98d7b49daec': {name:"NBA In the Zone 2000"                                                          },
  '9f4f05b3b59eb696': {name:"NBA In the Zone 2000"                                                          },
  '3019126a74c25c66': {name:"NBA In the Zone '98"                                                           },
  '9bbaac36944d8df2': {name:"NBA In the Zone '98"                                                           },
  '4f5292a2492a6c3d': {name:"NBA In the Zone '99"                                                           },
  'dba8eeeb3cb2ecf2': {name:"NBA Jam 2000"                                                                  },
  'a0cad0b6c893f4e3': {name:"NBA Jam 2000"                                                                  },
  'f6290781c1cf3fe0': {name:"NBA Jam 99"                                                                    },
  '1e8300e6a822f459': {name:"NBA Jam 99"                                                                    },
  'eeb0255fdbc12762': {name:"NBA Live 2000"                                                                 },
  '8f9c49ebb66745cd': {name:"NBA Live 2000"                                                                 },
  '9b1cf85735fa3311': {name:"NBA Live 99"                                                                   },
  '5ff484cfebf6e400': {name:"NBA Live 99"                                                                   },
  '2f96deac7ff8cbb2': {name:"NBA Pro 98"                                                                    },
  'b680178d76b9b357': {name:"NBA Pro 99"                                                                    },
  'f480fe3f7e5fc1a7': {name:"NBA Showtime - NBA on NBC"                                                     },
  'db0e7e142cb1c536': {name:"Neon Genesis Evangelion",                                    save:'Eeprom16k'  },
  '4fd5ea30f60b6231': {name:"NFL Blitz - Special Edition"                                                   },
  '70b194d0ccb5c4d7': {name:"NFL Blitz"                                                                     },
  '6909a01585a2e534': {name:"NFL Blitz 2000"                                                                },
  'eb35fa36362e5ee8': {name:"NFL Blitz 2001"                                                                },
  'd04eabe3d20d0483': {name:"NFL Quarterback Club 2000"                                                     },
  '9e5abd88bfdf1fe8': {name:"NFL Quarterback Club 2000"                                                     },
  '224678288529b2ff': {name:"NFL Quarterback Club 2001"                                                     },
  'f8e29bd8df7ac999': {name:"NFL Quarterback Club 98"                                                       },
  'f49e624b9b1db299': {name:"NFL Quarterback Club 98"                                                       },
  'ffed76be092d4520': {name:"NFL Quarterback Club 99"                                                       },
  '47cfa352fc3bc14e': {name:"NFL Quarterback Club 99"                                                       },
  '6e801a591d92e6a5': {name:"NHL 99"                                                                        },
  '1e607d28aef8f4ab': {name:"NHL 99"                                                                        },
  '30dcef8261246a80': {name:"NHL Blades of Steel '99"                                                       },
  'c3cdfd6dc801e74d': {name:"NHL Breakaway 98"                                                              },
  '9276ce297985c571': {name:"NHL Breakaway 98"                                                              },
  'd06817444ff2737d': {name:"NHL Breakaway 99"                                                              },
  'cb21468727c13100': {name:"NHL Breakaway 99"                                                              },
  'd95c89a96c012070': {name:"NHL Pro 99"                                                                    },
  '4d675728da3743cc': {name:"Nightmare Creatures"                                                           },
  'df3c3ccdfa937731': {name:"Nintama Rantarou 64"                                                           },
  '45b8508f2fd229d7': {name:"Nuclear Strike 64"                                                             },
  'bbdd9849bcaeb7f7': {name:"Nuclear Strike 64"                                                             },
  '97a1978ac1f62d27': {name:"Nuclear Strike 64"                                                             },
  '20b93bd8166440cc': {name:"Nushi Zuri 64"                                                                 },
  '18169b5b49c6431b': {name:"Nushi Tsuri 64 - Shiokaze ni Notte"                                            },
  'ec939031ef09c20f': {name:"Off Road Challenge"                                                            },
  'd08922819632e5c2': {name:"Off Road Challenge"                                                            },
  'c59b41e6e31d0169': {name:"Ogre Battle 64 - Person of Lordly Caliber",                  save:'SRAM'       },
  '67cf7503aa3fa956': {name:"Ogre Battle 64 - Person of Lordly Caliber (v1.1)",           save:'SRAM'       },
  '6910969c8d48eaf5': {name:"Ohzumou 64",                                                 save:'Eeprom4k'   },
  '168bc185af2296df': {name:"Ohzumou 64 2",                                               save:'Eeprom4k'   },
  '669464dcd9f02f57': {name:"Onegai Monsters"                                                               },
  '3b4b5574b5bcaef4': {name:"Pachinko 365 Nichi"                                                            },
  '3ae5ee653c737ded': {name:"Paper Mario",                                                save:'FlashRam'   },
  'af29ab1928cd1bc7': {name:"Paper Mario",                                                save:'FlashRam'   },
  '129b52893bed5308': {name:"Paperboy"                                                                      },
  '9e8d193e7e26e1f2': {name:"Paperboy"                                                                      },
  '386b97ac7ac9a9c3': {name:"Paperboy"                                                                      },
  '31cbe2cf1d1e6b4d': {name:"Parlor! Pro 64 Pachinko Jikki Simulation Game"                                 },
  '6647dd6ea89b3ea9': {name:"Pawapuro Puroyakyu 2001"                                                       },
  'ad5b4934269d2e50': {name:"Pawapuro Puroyakyu 4"                                                          },
  'da4329d2c0772bac': {name:"Pawapuro Puroyakyu 5"                                                          },
  'b7205eb7fdfdfeb3': {name:"Pawapuro Puroyakyu 6"                                                          },
  '8c1168f44ee42ee3': {name:"PD Ultraman Battle Collection 64",                           save:'Eeprom16k'  },
  'fbb1ab739360ca9c': {name:"Penny Racers",                                               save:'Eeprom4k'   },
  '83eb3cc81962c5fd': {name:"Penny Racers",                                               save:'Eeprom4k'   },
  'b47e749643b24b10': {name:"Perfect Dark",                                               save:'Eeprom16k'  },
  '0780b0e433ff02a6': {name:"Perfect Dark",                                               save:'Eeprom16k'  },
  'cc60f4ddc034a63c': {name:"Perfect Dark (v1.0)",                                        save:'Eeprom16k'  },
  '8fb9f24166b458b4': {name:"Perfect Dark (v1.1)",                                        save:'Eeprom16k'  },
  '81e84cb52661cbbc': {name:"PGA European Tour",                                          save:'Eeprom4k'   },
  '02c608eea6d5c26b': {name:"PGA European Tour",                                          save:'Eeprom4k'   },
  '0553243faa740bfc': {name:"Pikachu Genki Dechu"                                                           },
  '1c9651c8faaafc78': {name:"Pilot Wings 64",                                             save:'Eeprom4k'   },
  '0148cc0991e42ee4': {name:"Pilot Wings 64",                                             save:'Eeprom4k'   },
  'd55aa01a802df546': {name:"Pilot Wings 64",                                             save:'Eeprom4k'   },
  '0d690fec8c43a732': {name:"Pocket Monsters Snap",                                       save:'FlashRam'   },
  '59825e661dbd98d0': {name:"Pocket Monsters Stadium",                                    save:'FlashRam'   },
  '865877637b0eb85f': {name:"Pocket Monsters Stadium 2",                                  save:'FlashRam'   },
  'ac47477a23ecee44': {name:"Pokemon Puzzle League",                                      save:'FlashRam'   },
  'a753c519524b0fa7': {name:"Pokemon Puzzle League",                                      save:'FlashRam'   },
  'f3e6b23efe9e2f06': {name:"Pokemon Puzzle League",                                      save:'FlashRam'   },
  '53d11c4af8ae30d8': {name:"Pokemon Puzzle League",                                      save:'FlashRam'   },
  '6f97f54fd859f5ac': {name:"Pokemon Snap",                                               save:'FlashRam'   },
  '6a287d81167441ef': {name:"Pokemon Snap",                                               save:'FlashRam'   },
  '0d7253574d888a2a': {name:"Pokemon Snap",                                               save:'FlashRam'   },
  '47b512cae44efa71': {name:"Pokemon Snap",                                               save:'FlashRam'   },
  '3a296cba38a3af9f': {name:"Pokemon Snap",                                               save:'FlashRam'   },
  '4650c8c0051b0561': {name:"Pokemon Snap",                                               save:'FlashRam'   },
  '408db17b59851383': {name:"Pokemon Snap",                                               save:'FlashRam'   },
  '729811399f2e7207': {name:"Pokemon Snap Station",                                       save:'FlashRam'   },
  '432d121a0faf7dc1': {name:"Pokemon Stadium - Kiosk",                                    save:'FlashRam'   },
  '2da83fa55dc1e2da': {name:"Pokemon Stadium",                                            save:'FlashRam'   },
  '5de0c991b9af3aad': {name:"Pokemon Stadium",                                            save:'FlashRam'   },
  '1b1e0142b52d55e3': {name:"Pokemon Stadium",                                            save:'FlashRam'   },
  'b3d9f590f0dc0e9d': {name:"Pokemon Stadium",                                            save:'FlashRam'   },
  'a35335a2392dbf42': {name:"Pokemon Stadium",                                            save:'FlashRam'   },
  '757207849c5b3157': {name:"Pokemon Stadium",                                            save:'FlashRam'   },
  'ce49e5b6c03481dc': {name:"Pokemon Stadium",                                            save:'FlashRam'   },
  '821157036dd02f89': {name:"Pokemon Stadium 2",                                          save:'FlashRam'   },
  'c7a55aacc3cdb0a9': {name:"Pokemon Stadium 2",                                          save:'FlashRam'   },
  '727e9b435d49a1c1': {name:"Pokemon Stadium 2",                                          save:'FlashRam'   },
  '0bfca1d04b07b82f': {name:"Pokemon Stadium 2",                                          save:'FlashRam'   },
  '00afceef48480922': {name:"Pokemon Stadium 2",                                          save:'FlashRam'   },
  '5bfca1d04b07b82f': {name:"Pokemon Stadium 2",                                          save:'FlashRam'   },
  '9c365229a8c3e4b6': {name:"Pokemon Stadium 2",                                          save:'FlashRam'   },
  'c2d74fee38d9f19c': {name:"Pokemon Stadium GS",                                         save:'FlashRam'   },
  '9207384145e067a1': {name:"Polaris Sno Cross"                                                             },
  'fadca6d7b7b6fecc': {name:"Power League Baseball 64"                                                      },
  'ad5789cfa97ed596': {name:"Power Rangers - Lightspeed Rescue"                                             },
  '110cf6397dba2eab': {name:"Power Rangers - Lightspeed Rescue"                                             },
  '75d474fcab78029a': {name:"Powerpuff Girls - Chemical X-Traction",                      save:'Eeprom4k'   },
  '547fd2f3f9ac11c1': {name:"Premier Manager 64"                                                            },
  '4e0ca19bd3ab0804': {name:"Pro Mahjong Kiwame 64"                                                         },
  '0fb3dc1b76d832a1': {name:"Pro Mahjong Tsuwamono 64"                                                      },
  '8366ce8a6e42ba3f': {name:"Pro Yak Yu King Baseball"                                                      },
  'b972464be75dbe2d': {name:"Puyo Puyo 4 - Puyo Puyo Party",                              save:'Eeprom4k'   },
  '6b7e8094e462cc60': {name:"Puyo Puyo Sun 64",                                           save:'Eeprom4k'   },
  '4707dec0d372dfa2': {name:"Puzzle Bobble 64"                                                              },
  '9cf75b9fa008fed2': {name:"Quake 64"                                                                      },
  '741d9316346ddc65': {name:"Quake 64"                                                                      },
  '43f1a8bd622dafb1': {name:"Quake II"                                                                      },
  'd7d93374d022432c': {name:"Quake II"                                                                      },
  'd94dbbc80b435fcc': {name:"Quest 64"                                                                      },
  'a55f70280e6909b5': {name:"Racing Simulation - Monaco Grand Prix",                      save:'Eeprom4k'   },
  '2dac77289a13dcc3': {name:"Racing Simulation 2"                                                           },
  'f0ca1e9f0e8ac4ee': {name:"Rakuga Kids"                                                                   },
  '6818d267614042c5': {name:"Rakuga Kids"                                                                   },
  '0cbad935865548df': {name:"Rally '99"                                                                     },
  '3d8ea87371c5c53a': {name:"Rally Challenge 2000",                                       save:'Eeprom4k'   },
  'e4f99fc27dfe4b26': {name:"Rampage - World Tour"                                                          },
  '4844d484b019ca67': {name:"Rampage - World Tour"                                                          },
  '9b093d67de08c8a4': {name:"Rampage 2 - Universal Tour"                                                    },
  '4942fc5d079c5299': {name:"Rampage 2 - Universal Tour"                                                    },
  '8ec40403b80140ac': {name:"Rat Attack"                                                                    },
  'f10bfd20871dcff5': {name:"Rat Attack"                                                                    },
  '9bbfc5f3e2330f16': {name:"Rayman 2 - The Great Escape"                                                   },
  '0be1d56046eded8b': {name:"Rayman 2 - The Great Escape"                                                   },
  '4a831839290cb515': {name:"Razor Freestyle Scooter"                                                       },
  '29b4b7ea572cc9ba': {name:"Ready 2 Rumble Boxing"                                                         },
  '34a3d48b058b131e': {name:"Ready 2 Rumble Boxing"                                                         },
  '339521e9bdaffb13': {name:"Ready 2 Rumble Boxing Round 2"                                                 },
  'a5b118aaeb6adb07': {name:"Resident Evil 2",                                            save:'SRAM'       },
  '8e0e509bb35005e9': {name:"Resident Evil 2",                                            save:'SRAM'       },
  '87a91f0fa6afc1bf': {name:"Re-Volt"                                                                       },
  '9ee2e7c3cc51725d': {name:"Re-Volt"                                                                       },
  '6c7450f00b827b24': {name:"Road Rash 64"                                                                  },
  '6a36d8029cefab6c': {name:"Road Rash 64"                                                                  },
  'db4d6b0b82e67196': {name:"Roadsters"                                                                     },
  '707ae874d4ae9362': {name:"Roadsters"                                                                     },
  'd3a5bc747e5f944a': {name:"Roadsters Trophy"                                                              },
  '0f692b27777a0aad': {name:"Robot Ponkottsu 64 - Caramel of the 7 Seas",                 save:'Eeprom4k'   },
  '93b8a075b521a34c': {name:"Robotech - Crystal Dreams"                                                     },
  '324b8eac2673b4e7': {name:"Robotron 64"                                                                   },
  '4f9df69f59005f19': {name:"Robotron 64"                                                                   },
  'f875d39fc82df345': {name:"Rocket - Robot on Wheels",                                   save:'Eeprom4k'   },
  '85e05e0c3edd67a1': {name:"Rocket - Robot on Wheels",                                   save:'Eeprom4k'   },
  '3b5966d6075ca2d7': {name:"Rockman Dash",                                               save:'FlashRam'   },
  '7e260025cec37e2a': {name:"RR64 - Ridge Racer 64",                                      save:'Eeprom16k'  },
  '1070e9fea0a9944e': {name:"RR64 - Ridge Racer 64",                                      save:'Eeprom16k'  },
  '378f8f658dd21318': {name:"RTL World League Soccer 2000"                                                  },
  'c5b3020cb811259e': {name:"Rugrats - Scavenger Hunt"                                                      },
  'dadf3a4daefc9875': {name:"Rugrats - Treasure Hunt"                                                       },
  '0fca1a458abc6378': {name:"Rugrats - Treasure Hunt"                                                       },
  'b46c692bd8dc937b': {name:"Rugrats - Treasure Hunt"                                                       },
  '3215c21fd466640b': {name:"Rugrats in Paris - The Movie"                                                  },
  '391dc60aa603faab': {name:"Rugrats in Paris - The Movie"                                                  },
  '31e0d6ed13601368': {name:"Rush 2 - Extreme Racing USA"                                                   },
  '3621cfb715a70afa': {name:"Rush 2 - Extreme Racing USA"                                                   },
  'f34791760ec13320': {name:"S.C.A.R.S."                                                                    },
  '602d8e913e6865f8': {name:"S.C.A.R.S."                                                                    },
  '20186b2a66f4bc6a': {name:"San Francisco Rush - Extreme Racing"                                           },
  'b016d1610cd624fa': {name:"San Francisco Rush - Extreme Racing"                                           },
  'a2eca9b98ee4aa17': {name:"San Francisco Rush 2049"                                                       },
  '1894d251e36ab4d5': {name:"San Francisco Rush 2049"                                                       },
  'c44e810ca85cfe58': {name:"Scooby-Doo - Classic Creep Capers"                                             },
  '1d22bde3d334083c': {name:"Scooby-Doo - Classic Creep Capers"                                             },
  'b7f6f5eb39d756c9': {name:"SD Hiryuu no Ken Densetsu"                                                     },
  '75fdd584df3cfdbb': {name:"Shadow Man"                                                                    },
  'b560473a10d4742d': {name:"Shadow Man"                                                                    },
  'c3f806eaeddec207': {name:"Shadow Man"                                                                    },
  'e537c460e31e25a2': {name:"Shadow Man"                                                                    },
  'ce97680354fad4e0': {name:"Shadowgate 64 - Trials of the Four Towers"                                     },
  '556fb4020b8d7761': {name:"Shadowgate 64 - Trials of the Four Towers"                                     },
  '84ea4ed8b4f1b245': {name:"Shadowgate 64 - Trials of the Four Towers"                                     },
  'f2fcc12bf40d9a7b': {name:"Shadowgate 64 - Trials of the Four Towers"                                     },
  '0c28d5b12abca74b': {name:"Sim City 2000"                                                                 },
  'b00fbcb6982181e3': {name:"Sin and Punishment - Tsumi To Batsu",                        save:'Eeprom4k'   },
  '19d5f42e5e0c4ac6': {name:"Snow Speeder"                                                                  },
  '9deaf4dbc0823e33': {name:"Snowboard Kids"                                                                },
  'ff04fc84e93c25b1': {name:"Snowboard Kids"                                                                },
  'a0cdd75fad51bbd9': {name:"Snowboard Kids"                                                                },
  'ea290c93bf459293': {name:"Snowboard Kids 2",                                           save:'Eeprom4k'   },
  '1a1d75c2ff9bc1f8': {name:"Snowboard Kids 2",                                           save:'Eeprom4k'   },
  '512321224b594640': {name:"Sonic Wings Assault",                                        save:'Eeprom4k'   },
  '39e9cb7e9517333c': {name:"South Park"                                                                    },
  '6236b5209f89617b': {name:"South Park"                                                                    },
  '48a90cc04bd3608e': {name:"South Park - Chef's Luv Shack"                                                 },
  '76b2f307393d8fec': {name:"South Park Rally"                                                              },
  '3afc8a4ff22d91f7': {name:"South Park Rally"                                                              },
  '123446378d38c5ea': {name:"Space Dynamites",                                            save:'Eeprom4k'   },
  '9723feeb34da74ff': {name:"Space Invaders"                                                                },
  '8438e2bfafea48ef': {name:"Spacestation Silicon Valley",                                save:'Eeprom4k'   },
  '72e270fcaae7ff08': {name:"Spacestation Silicon Valley",                                save:'Eeprom4k'   },
  '71d10ea66ed0853d': {name:"Spider-Man"                                                                    },
  '6805ed0d5e510215': {name:"St. Andrews Old Course"                                                        },
  'f815d0a743aa8922': {name:"Star Fox 64",                                                save:'Eeprom4k'   },
  'c1a7caff37858568': {name:"Star Fox 64",                                                save:'Eeprom4k'   },
  'a00b78ba34db210f': {name:"Star Fox 64",                                                save:'Eeprom4k'   },
  '853cd9dde881e3da': {name:"Star Soldier Vanishing Earth",                               save:'Eeprom4k'   },
  '23eb03b73ae5aa28': {name:"Star Soldier Vanishing Earth",                               save:'Eeprom4k'   },
  '42a263f13b9b44f2': {name:"Star Twins"                                                                    },
  '124aa266c26aaf22': {name:"Star Wars - Rogue Squadron",                                 save:'Eeprom4k'   },
  'a2bd5c164622a605': {name:"Star Wars - Rogue Squadron",                                 save:'Eeprom4k'   },
  'c1919121613c1833': {name:"Star Wars - Rogue Squadron",                                 save:'Eeprom4k'   },
  'ec4ba2664fd9ad2e': {name:"Star Wars - Rogue Squadron",                                 save:'Eeprom4k'   },
  'bbe8e07eaa11e449': {name:"Star Wars - Rogue Squadron",                                 save:'Eeprom4k'   },
  '63ac24fe17aa411b': {name:"Star Wars - Shadows of the Empire",                          save:'Eeprom4k'   },
  '8166484d45927dab': {name:"Star Wars - Shadows of the Empire",                          save:'Eeprom4k'   },
  '5c7e4d2622468718': {name:"Star Wars - Shadows of the Empire (v1.0)",                   save:'Eeprom4k'   },
  '91b0474160102563': {name:"Star Wars - Shadows of the Empire (v1.1)",                   save:'Eeprom4k'   },
  '54edd74d7d28f974': {name:"Star Wars - Shadows of the Empire (v1.2)",                   save:'Eeprom4k'   },
  '90487e82dc688495': {name:"Star Wars - Shuggeki Rogue Chitai",                          save:'Eeprom4k'   },
  '9b98023de281a3d4': {name:"Star Wars Episode I - Battle for Naboo",                     save:'Eeprom4k'   },
  'e2ace6ea84430b02': {name:"Star Wars Episode I - Battle for Naboo",                     save:'Eeprom4k'   },
  '9803f7728ba95665': {name:"Star Wars Episode I - Racer",                                save:'Eeprom16k'  },
  '52b1f561ab226104': {name:"Star Wars Episode I - Racer",                                save:'Eeprom16k'  },
  'c42ded5302802506': {name:"Star Wars Episode I - Racer",                                save:'Eeprom16k'  },
  'fbfb8406a5a83e5d': {name:"StarCraft 64",                                               save:'FlashRam'   },
  'a35ecf42df34139a': {name:"StarCraft 64",                                               save:'FlashRam'   },
  'c32c9bbca54dd04e': {name:"StarCraft 64 (beta)",                                        save:'FlashRam'   },
  'b8a5ed9403e97386': {name:"Starshot - Space Circus Fever",                              save:'Eeprom4k'   },
  '550e9ed89aa97ab1': {name:"Starshot - Space Circus Fever",                              save:'Eeprom4k'   },
  'd7d81095d20d1035': {name:"Stunt Racer 64"                                                                },
  '696b64f4951075c5': {name:"Super B-Daman - Battle Phoenix 64"                                             },
  '5a211daa9abecb91': {name:"Super Bowling 64"                                                              },
  '85f3f2f37f0c496e': {name:"Super Bowling 64"                                                              },
  'f08cbca5f651bad8': {name:"Super Mario 64 - Star Road",                                 save:'Eeprom4k'   },
  '76d1fe61f9bb05ea': {name:"Super Mario 64 - The Missing Stars v2",                      save:'Eeprom4k'   },
  '88aefe21241410a4': {name:"Super Mario 64 - The Missing Stars",                         save:'Eeprom4k'   },
  'd5ba7f26dcfa9375': {name:"Super Mario 64 Ru",                                          save:'Eeprom4k'   },
  'c5425a63dc8ec5bd': {name:"Super Mario 64 Extended",                                    save:'Eeprom4k'   },
  'ff2b5a632623028b': {name:"Super Mario 64",                                             save:'Eeprom4k'   },
  '0e3daa4e247c7574': {name:"Super Mario 64",                                             save:'Eeprom4k'   },
  '36f03ca0d2c5c1bc': {name:"Super Mario 64",                                             save:'Eeprom4k'   },
  'a8a4fbd62caa2663': {name:"Super Mario 64 Shindou Edition",                             save:'Eeprom4k'   },
  '80205766e148e328': {name:"Super Robot Spirits"                                                           },
  '10d84916d2d63af7': {name:"Super Robot Taisen 64"                                                         },
  '5b8b6b91a4850b78': {name:"Super Smash Brothers",                                       save:'SRAM'       },
  '485f9493302e0f5c': {name:"Super Smash Brothers",                                       save:'SRAM'       },
  'a1fd26dde36b4acb': {name:"Super Smash Brothers",                                       save:'SRAM'       },
  '222ee09cb0f16e20': {name:"Super Speed Race 64"                                                           },
  '532545c1d9247b5d': {name:"Supercross 2000"                                                               },
  '7f12bb2cd8bfc209': {name:"Supercross 2000"                                                               },
  '5bf3e8a2d987dcc9': {name:"Superman - The Animated Series"                                                },
  '74ab4cb4299a0207': {name:"Superman - The Animated Series"                                                },
  'f311e83524277999': {name:"Susume! Taisen Puzzle Dama Toukon! Marumata Chou"                              },
  '56a48bb9af762b5b': {name:"Tamagotchi World 64" ,                                       save:'Eeprom4k'   },
  '54ddbcae4a83ff15': {name:"Taz Express",                                                save:'Eeprom4k'   },
  'bb2f04d97c99fffc': {name:"Telefoot Soccer 2000"                                                          },
  'a6db3a969bc8d5f7': {name:"Tetris 64"                                                                     },
  'beda1f3cbae0a402': {name:"Tetrisphere",                                                save:'Eeprom4k'   },
  'a984e60fc47ab78b': {name:"Tetrisphere",                                                save:'Eeprom4k'   },
  '161b7d91e19b1f83': {name:"The Legend of Zelda - OoT Nightmare Mod",                    save:'SRAM'       },
  'c7c7b1f831a4fc1e': {name:"The Legend of Zelda - Ocarina of Time Ru",                   save:'SRAM'       },
  '4fecec6a144892f0': {name:"The Legend of Zelda - Majora's Mask ",                       save:'FlashRam'   },
  'E585C39FC705CC3E': {name:"The Legend of Zelda - Majora's Mask Debug",                  save:'FlashRam'   },
  'b71170ec2bd71676': {name:"The Legend of Zelda - Ocarina of Time (v1.0)",               save:'SRAM'       },
  '1a0034f006ed47ae': {name:"The Legend of Zelda - Ocarina of Time Master Quest (GC)",    save:'SRAM'       },
  '69b544b085193c37': {name:"The Legend of Zelda - Ocarina of Time (v1.0)",               save:'SRAM'       },
  '1fa83dd4191e1e02': {name:"The Legend of Zelda - Ocarina of Time (v1.1)",               save:'SRAM'       },
  'bd5f05b20c4eab0b': {name:"The Legend of Zelda - Ocarina of Time (v1.1)",               save:'SRAM'       },
  'aea23b699f4ef1b7': {name:"The Legend of Zelda - Ocarina of Time (v1.2)",               save:'SRAM'       },
  'c35a46091b50cbf8': {name:"The Legend of Zelda - Ocarina of Time (GC)",                 save:'SRAM'       },
  'f6187d915354bc69': {name:"The Legend of Zelda - Zelda's Birthday",                     save:'SRAM'       },
  '08eb43b49311b34d': {name:"The Legend of Zelda - Majora's Mask (GC)",                   save:'FlashRam'   },
  '459379bf027aff39': {name:"The Legend of Zelda - Majora's Mask (Demo)",                 save:'FlashRam'   },
  '1c635453f0dea203': {name:"The Legend of Zelda - Majora's Mask",                        save:'FlashRam'   },
  'c65579e9388d33bc': {name:"The Legend of Zelda - Majora's Mask",                        save:'FlashRam'   },
  'ba35ddf375e05241': {name:"The Legend of Zelda - Ocarina of Time (GC)",                 save:'SRAM'       },
  'f336411da9ee63af': {name:"The Legend of Zelda - OOT (Master Quest)",                   save:'SRAM'       },
  '3f14532151632d99': {name:"The New Tetris",                                             save:'SRAM'       },
  '0aff1ce6710d1cce': {name:"The New Tetris",                                             save:'SRAM'       },
  '33ddbf4e849d4c66': {name:"Tigger's Honey Hunt",                                        save:'FlashRam'   },
  '2ff7c4e006159e76': {name:"Tigger's Honey Hunt",                                        save:'FlashRam'   },
  '1c39e76333eacce6': {name:"Tom and Jerry in Fists of Furry",                            save:'Eeprom4k'   },
  'fb4e4f2bfe11c543': {name:"Tom and Jerry in Fists of Furry",                            save:'Eeprom4k'   },
  '3329418ddb648f58': {name:"Tom Clancy's Rainbow Six"                                                      },
  '35f36b4881cc4d03': {name:"Tom Clancy's Rainbow Six"                                                      },
  '420c2a397de790b7': {name:"Tom Clancy's Rainbow Six"                                                      },
  '3daf7548a2d3669a': {name:"Tom Clancy's Rainbow Six"                                                      },
  '14979eef7d2c3bc0': {name:"Tonic Trouble"                                                                 },
  '6e913f0998b60844': {name:"Tonic Trouble"                                                                 },
  '22c04e2085d119b1': {name:"Tony Hawk's Pro Skater (v1.0)"                                                 },
  '804114e0c9780b65': {name:"Tony Hawk's Pro Skater (v1.1)"                                                 },
  'a526899f09b48705': {name:"Tony Hawk's Pro Skater"                                                        },
  '57b5ea840f198ac8': {name:"Tony Hawk's Pro Skater 2"                                                      },
  '180e1599a5e66612': {name:"Tony Hawk's Pro Skater 2"                                                      },
  '99150E181266E6A5': {name:"Tony Hawk's Pro Skater 2"                                                      },
  'b5707f1afdb9b700': {name:"Tony Hawk's Pro Skater 3"                                                      },
  'bf6e749a99ea0228': {name:"Toon Panic (Proto)"                                                            },
  'f002cc8e81de8b7f': {name:"Top Gear Hyper-Bike",                                        save:'Eeprom4k'   },
  'c6493f5fb014c70d': {name:"Top Gear Hyper-Bike",                                        save:'Eeprom4k'   },
  '80cd41d712b9a9ac': {name:"Top Gear Overdrive",                                         save:'Eeprom4k'   },
  '4ff2780517bf7591': {name:"Top Gear Overdrive",                                         save:'Eeprom4k'   },
  '38a59bd089541a1c': {name:"Top Gear Overdrive",                                         save:'Eeprom4k'   },
  '3d9b2662e8b111fe': {name:"Top Gear Rally",                                             save:'Eeprom4k'   },
  '4762590e8b4b3d75': {name:"Top Gear Rally",                                             save:'Eeprom4k'   },
  '01e7437fd1286353': {name:"Top Gear Rally",                                             save:'Eeprom4k'   },
  'e07359beb8edb089': {name:"Top Gear Rally 2"                                                              },
  'd62cefcfe673e9c9': {name:"Top Gear Rally 2"                                                              },
  '77b6babee4b5b051': {name:"Top Gear Rally 2"                                                              },
  '77d32df847b33f8c': {name:"Top Gear Rally 2"                                                              },
  'a43c70efc99a4a4d': {name:"Toukon Road - Brave Spirits"                                                   },
  '437f1c551c834991': {name:"Toukon Road 2 - The Next Generation"                                           },
  '75902a781d6352e5': {name:"Toy Story 2"                                                                   },
  '3e7450a1cd2225cf': {name:"Toy Story 2"                                                                   },
  '97db93cbd5635c7f': {name:"Toy Story 2"                                                                   },
  '5838ebcc972d9526': {name:"Toy Story 2"                                                                   },
  '436b4bfea7291d08': {name:"Triple Play 2000"                                                              },
  '0ffae466d0c788de': {name:"Turok - Legenden des Verlorenen Landes"                                        },
  '8b49b9ad558ff2da': {name:"Turok - Rage Wars"                                                             },
  '1462a21e0f9090e7': {name:"Turok - Rage Wars"                                                             },
  'dd09702f53ac3bfc': {name:"Turok - The Dinosaur Hunter"                                                   },
  '63d95f661266ccb5': {name:"Turok - The Dinosaur Hunter"                                                   },
  'b8e66a9122ab1788': {name:"Turok - The Dinosaur Hunter"                                                   },
  '0df1702fff87415c': {name:"Turok - The Dinosaur Hunter (v1.0)"                                            },
  'dd095f6653ac3bfc': {name:"Turok - The Dinosaur Hunter (v1.0)"                                            },
  'cd0d702fc9c56c17': {name:"Turok - The Dinosaur Hunter (v1.1/v1.2)"                                       },
  '49770e2e599db4b8': {name:"Turok 2 - Seeds of Evil"                                                       },
  '942bb9e025e0a7b9': {name:"Turok 2 - Seeds of Evil"                                                       },
  '0b8405fe0c329393': {name:"Turok 2 - Seeds of Evil"                                                       },
  '118a08497e959464': {name:"Turok 2 - Seeds of Evil"                                                       },
  '942bb9e0bd7ce880': {name:"Turok 2 - Seeds of Evil"                                                       },
  'fc5ac9e8da21d135': {name:"Turok 2 - Seeds of Evil - Kiosk"                                               },
  'f179a589ef977e66': {name:"Turok 3 - Shadow of Oblivion"                                                  },
  'f22f166a4c709320': {name:"Turok 3 - Shadow of Oblivion"                                                  },
  '329dc9bb80aa7d11': {name:"Twisted Edge Extreme Snowboarding"                                             },
  'b8a588e6183f4bb1': {name:"Twisted Edge Extreme Snowboarding"                                             },
  '7851da6ec1fe96d3': {name:"Vigilante 8"                                                                   },
  '6a0571ea474821e4': {name:"Vigilante 8"                                                                   },
  'a282bce294d61c59': {name:"Vigilante 8"                                                                   },
  'f4791f15e5c8ed8e': {name:"Vigilante 8"                                                                   },
  '6d86c5f5d9132705': {name:"Vigilante 8 - 2nd Offense"                                                     },
  '7ebc10dd51b300f9': {name:"Vigilante 8 - 2nd Offense"                                                     },
  '986c006081a30526': {name:"Violence Killer - Turok New Generation"                                        },
  '8b24b3824d243ee7': {name:"Virtual Chess 64"                                                              },
  '21a2da2fcea788a5': {name:"Virtual Chess 64"                                                              },
  '43764a4ed73974a3': {name:"Virtual Pool 64"                                                               },
  'd0f2f9989cf0d903': {name:"Virtual Pool 64"                                                               },
  'c4085c048b79fd4a': {name:"Virtual Pro Wrestling",                                      save:'Eeprom4k'   },
  '354209cd624b0788': {name:"Virtual Pro Wrestling 2",                                    save:'SRAM'       },
  '3890053c8221bfc8': {name:"V-Rally Edition 99",                                         save:'Eeprom4k'   },
  'a524024d9457eb1b': {name:"V-Rally Edition 99",                                         save:'Eeprom4k'   },
  '196b6e635fdc7de5': {name:"V-Rally Edition 99",                                         save:'Eeprom4k'   },
  '8ad56680c1cadec3': {name:"Waialae Country Club - True Golf Classics",                  save:'Eeprom4k'   },
  '75300593430f1e26': {name:"Waialae Country Club - True Golf Classics",                  save:'Eeprom4k'   },
  'ad57500c6e126e04': {name:"Waialae Country Club - True Golf Classics (1.1)",            save:'Eeprom4k'   },
  'e28c31ddba9837b7': {name:"Waialae Country Club - True Golf Classics (1.1)",            save:'Eeprom4k'   },
  'f628fef7c3acf2c3': {name:"War Gods"                                                                      },
  '70cc15d7d6f51c27': {name:"War Gods"                                                                      },
  'd691915c06c30ab3': {name:"Wave Race 64",                                               save:'Eeprom4k'   },
  '96fa0e65a7f9dd30': {name:"Wave Race 64",                                               save:'Eeprom4k'   },
  '531fe17d9d2f8774': {name:"Wave Race 64 (v1.0)",                                        save:'Eeprom4k'   },
  '614b2f496a14e504': {name:"Wave Race 64 (v1.1)",                                        save:'Eeprom4k'   },
  'e2f35d53f1899760': {name:"Wave Race 64 Shindou Edition",                               save:'Eeprom4k'   },
  '59aa3bdc6a45bb0a': {name:"Wayne Gretzky's 3D Hockey v1.1"                                                },
  '3f22456b565c0ef0': {name:"Wayne Gretzky's 3D Hockey"                                                     },
  '431030f11a5480fd': {name:"Wayne Gretzky's 3D Hockey"                                                     },
  '4b090922af59952c': {name:"Wayne Gretzky's 3D Hockey"                                                     },
  '59389d5a10e7aa97': {name:"Wayne Gretzky's 3D Hockey '98"                                                 },
  'f3451b666d26d69e': {name:"Wayne Gretzky's 3D Hockey '98"                                                 },
  'dd5a6f39a7ec9366': {name:"WCW Backstage Assault"                                                         },
  'd68cbe33126918ec': {name:"WCW Mayhem"                                                                    },
  '58067baa7b93969c': {name:"WCW Mayhem"                                                                    },
  '1a5ac4d45eb225f4': {name:"WCW Nitro"                                                                     },
  '68afdb8b364b5b34': {name:"WCW vs. NWO - World Tour"                                                      },
  'bd193e2c5eee1351': {name:"WCW vs. NWO - World Tour (v1.0)"                                               },
  'b960be713cfbdb1d': {name:"WCW vs. NWO - World Tour (v1.2)"                                               },
  'ab96e5dee77a3baf': {name:"WCW-NWO Revenge",                                            save:'SRAM'       },
  '75a8e86886a4e70c': {name:"WCW-NWO Revenge",                                            save:'SRAM'       },
  '4fb5a8ce03d5217f': {name:"Wetrix"                                                                        },
  'faeab6dca3cfbbc6': {name:"Wetrix"                                                                        },
  '2b0996e84e4d24dc': {name:"Wheel of Fortune"                                                              },
  'c7c4eb0c32e99c0c': {name:"Wild Choppers",                                              save:'Eeprom4k'   },
  '7e9598edacdc4282': {name:"WinBack - Covert Operations"                                                   },
  'e056a01f6a94b9a4': {name:"WinBack - Covert Operations"                                                   },
  'af8c89d55bb60760': {name:"WinBack - Covert Operations"                                                   },
  '32272d1318910ec7': {name:"Wipeout 64"                                                                    },
  '7d0e3154d830546b': {name:"Wipeout 64"                                                                    },
  '65973ce4bec1b105': {name:"Wonder Project J2"                                                             },
  '38594b3a9d986fa2': {name:"Wonder Project J2 (English)"                                                   },
  'f7881e4f963f5a4a': {name:"Wonder Project J2 (English)"                                                   },
  '6a6d63bdba541f5d': {name:"World Cup 98"                                                                  },
  '9030fcf9c24e01ff': {name:"World Cup 98"                                                                  },
  'c8fe8d30f6b52ece': {name:"World Driver Championship"                                                     },
  '782706acb8fcaddf': {name:"World Driver Championship"                                                     },
  'a059e913b0ca930e': {name:"Worms - Armageddon",                                         save:'Eeprom4k'   },
  '7bc5212d8cc5e48f': {name:"Worms - Armageddon",                                         save:'Eeprom4k'   },
  '0fec5bcd0810fd86': {name:"WWF - War Zone"                                                                },
  'a475a233594450b8': {name:"WWF - War Zone"                                                                },
  'c851318f45f53a4f': {name:"WWF Attitude"                                                                  },
  '142fbed288374538': {name:"WWF Attitude"                                                                  },
  '7b5bf45be8ee6b59': {name:"WWF Attitude"                                                                  },
  '40064b4efbbc491b': {name:"WWF No Mercy",                                               save:'FlashRam'   },
  '8ef08d6dcfc308d0': {name:"WWF No Mercy (v1.0)",                                        save:'FlashRam'   },
  'c294db8cf0c646cb': {name:"WWF No Mercy (v1.1)",                                        save:'FlashRam'   },
  '0390a59064980831': {name:"WWF WrestleMania 2000",                                      save:'SRAM'       },
  'a57d731259919623': {name:"WWF Wrestlemania 2000",                                      save:'SRAM'       },
  'be5313c7eea609aa': {name:"WWF WrestleMania 2000",                                      save:'SRAM'       },
  '9dae5305c1e0d8ea': {name:"Xena Warrior Princess - Talisman of Fate"                                      },
  'c767160aa6463329': {name:"Xena Warrior Princess - Talisman of Fate"                                      },
  'c3968b9fdc9411a0': {name:"Yakouchuu II - Satsujun Kouru"                                                 },
  'e8d83723ec7c8e6b': {name:"Yoshi's Story",                                              save:'Eeprom16k'  },
  '60cacf2d47b15483': {name:"Yoshi's Story",                                              save:'Eeprom16k'  },
  '497df9d35b132469': {name:"Yoshi's Story",                                              save:'Eeprom16k'  },
  '8d3bda777c0d2b16': {name:"Yosuke no Mahjong Juku",                                     save:'Eeprom4k'   },
  '2d16e69f37407ee9': {name:"Yuke Yuke!! Trouble Makers"                                                    },
  'c1d0738466061223': {name:"Zelda no Densetsu 2 - Mujura no Kamen",                      save:'FlashRam'   },
  'b82df5f736e69521': {name:"Zelda no Densetsu - Toki no Ocarina Collectors Ed.",         save:'SRAM'       },
  'baf411f65c1384c5': {name:"Zelda no Densetsu - Toki no Ocarina (GC)",                   save:'SRAM'       },
  'ba453bf46f9b0e2f': {name:"Zelda no Densetsu - Toki no Ocarina (GC)",                   save:'SRAM'       },
  '127341ec5fde31eb': {name:"Zelda no Densetsu 2 - Mujura no Kamen",                      save:'FlashRam'   },
  '3804ae69f3f3632c': {name:"Zelda no Densetsu 2 - Mujura no Kamen (1.1)",                save:'FlashRam'   },
  'cd0c011cfab7d322': {name:"Zool - Majou Tsukai Densetsu"                                                  },
  '54a43977662af5a2': {name:"A HORiZON64 RELEASE"                                                           },
  '91d8ab5ccef62962': {name:"Fishy by NaN v0.3141"                                                          },
  'bc069a1355006b41': {name:"Mind Present-Readme"                                                           },
  'b1cb11c87c61b78f': {name:"Mind Present / DNX"                                                            },
  '54f53fcb0bcd7387': {name:"Tron Demo"                                                                     },
  '4549049c0c0b1ed3': {name:"POM99 - README"                                                                },
  '7a7d3b994df0542e': {name:"POM99 - MADEIRAGAMES"                                                          },
  '575aa9a37dc2e69f': {name:"POM99 - WET DREAMS"                                                            },
  '95c03bca0a864996': {name:"POM99 - CAN-BETA"                                                              },
  'd44a7da4f981fa8b': {name:"Manic Miner 64"                                                                },
  '2d369c9d086be65b': {name:"Shag-a-delic/CAMELOT"                                                          },
  'c479518fdc263580': {name:"Space Rotator"                                                                 },
  '70bb552625997d66': {name:"O.D.T.",                                                     save:'Controller' },
  'a61564e8535b3998': {name:"O.D.T.",                                                     save:'Controller' }
};
